@@ -1,12 +1,15 @@
 """Utility functions."""
 
 from datetime import datetime, date
+import importlib
 import json
+from pkg_resources import iter_entry_points
 
 import yaml
 from sqlalchemy.sql import exists
 
-from dtoolcore.utils import DEFAULT_CONFIG_PATH as CONFIG_PATH
+import pymongo.errors
+
 import dtoolcore.utils
 
 from dtool_lookup_server import (
@@ -24,6 +27,7 @@ from dtool_lookup_server.sql_models import (
     BaseURI,
     Dataset,
 )
+from dtool_lookup_server.config import Config
 
 DATASET_INFO_REQUIRED_KEYS = (
     "uuid",
@@ -39,6 +43,21 @@ DATASET_INFO_REQUIRED_KEYS = (
     "tags",
 )
 
+VALID_MONGO_QUERY_KEYS = (
+    "free_text",
+    "creator_usernames",
+    "base_uris",
+    "uuids",
+    "tags",
+)
+
+MONGO_QUERY_LIST_KEYS = (
+    "creator_usernames",
+    "base_uris",
+    "uuids",
+    "tags",
+)
+
 
 #############################################################################
 # Private helper functions.
@@ -59,29 +78,27 @@ def _get_base_uri_obj(base_uri):
 
 
 def _dict_to_mongo_query(query_dict):
-    valid_keys = ["free_text", "creator_usernames", "base_uris", "tags"]
-    list_keys = ["creator_usernames", "base_uris", "tags"]
 
     def _sanitise(query_dict):
         for key in list(query_dict.keys()):
-            if key not in valid_keys:
+            if key not in VALID_MONGO_QUERY_KEYS:
                 del query_dict[key]
-        for lk in list_keys:
+        for lk in MONGO_QUERY_LIST_KEYS:
             if lk in query_dict:
                 if len(query_dict[lk]) == 0:
                     del query_dict[lk]
 
-    def _deal_with_possible_or_statment(l, key):
-        if len(l) == 1:
-            return {key: l[0]}
+    def _deal_with_possible_or_statment(a_list, key):
+        if len(a_list) == 1:
+            return {key: a_list[0]}
         else:
-            return {"$or": [{key: v} for v in l]}
+            return {"$or": [{key: v} for v in a_list]}
 
-    def _deal_with_possible_and_statement(l, key):
-        if len(l) == 1:
-            return {key: l[0]}
+    def _deal_with_possible_and_statement(a_list, key):
+        if len(a_list) == 1:
+            return {key: a_list[0]}
         else:
-            return {key: {"$all": l}}
+            return {key: {"$all": a_list}}
 
     _sanitise(query_dict)
 
@@ -102,6 +119,13 @@ def _dict_to_mongo_query(query_dict):
                 "base_uri"
             )
         )
+    if "uuids" in query_dict:
+        sub_queries.append(
+            _deal_with_possible_or_statment(
+                query_dict["uuids"],
+                "uuid"
+            )
+        )
     if "tags" in query_dict:
         sub_queries.append(
             _deal_with_possible_and_statement(
@@ -122,17 +146,44 @@ def _dict_to_mongo_query(query_dict):
 # Generally useful dtool helper functions.
 #############################################################################
 
-# TODO: this function should probably live in  dtoolcore along with a test.
-def iter_datasets_in_base_uri(base_uri):
-    """Yield frozen datasets in a base URI."""
-    base_uri = dtoolcore.utils.sanitise_uri(base_uri)
-    StorageBroker = dtoolcore._get_storage_broker(base_uri, CONFIG_PATH)
-    for uri in StorageBroker.list_dataset_uris(base_uri, CONFIG_PATH):
-        try:
-            dataset = dtoolcore.DataSet.from_uri(uri)
-            yield dataset
-        except dtoolcore.DtoolCoreTypeError:
-            pass
+def config_to_dict(username):
+    # Authenticate the user.
+    get_user_obj(username)
+
+    core_config = Config.to_dict()
+    plugin_config = {}
+
+    # Iterate over all registered blueprints
+    # and get per-plugin configs if implemented.
+    # All plugins are expected to be top-level modules.
+    for ep in iter_entry_points("dtool_lookup_server.blueprints"):
+        module_name = ep.module_name.split(".")[0]
+        if module_name not in plugin_config:
+            try:
+                plugin_module = importlib.import_module(module_name)
+            except ImportError as exc:
+                # plugin import failed, this should not happen
+                plugin_config[module_name] = str(exc)
+                continue
+
+            try:
+                plugin_config[
+                    module_name] = plugin_module.config.Config.to_dict()
+            except AttributeError as exc:
+                # plugin did not implement config.Config.to_dict properly
+                plugin_config[module_name] = str(exc)
+                continue
+
+    # check for overlap between core config keys and plugin names
+    if len(set(core_config.keys()) & set(plugin_config.keys())) > 0:
+        raise ValueError(
+            "Plugin module names and core server config keys must not overlap."
+        )
+
+    all_config = core_config
+    if len(plugin_config) > 0:
+        all_config.update(plugin_config)
+    return all_config
 
 
 def generate_dataset_info(dataset, base_uri):
@@ -259,20 +310,8 @@ def list_datasets_by_user(username):
     return datasets
 
 
-def search_datasets_by_user(username, query):
-    """Search the datasets the user has access to.
-
-    Valid keys for the query are: creator_usernames, base_uris, free_text.  If
-    the query dictionary is empty all datasets, that a user has access to, are
-    returned.
-
-    :param username: username
-    :param query: dictionary specifying query
-    :returns: List of dicts if user is valid and has access to datasets.
-              Empty list if user is valid but has not got access to any
-              datasets.
-    :raises: AuthenticationError if user is invalid.
-    """
+def _preprocess_privileges(username, query):
+    """Preprocess a query dict according to per-user privileges."""
     user = get_user_obj(username)
 
     # Deal with base URIs. If not specified on the query add the ones that the
@@ -288,8 +327,36 @@ def search_datasets_by_user(username, query):
         ]
         query["base_uris"] = selected_uris
 
-    # If there are no base URIs at this point it means that the user has not
-    # got privileges to search for anything.
+    return query
+
+
+def preprocess_query_base_uris(username, query):
+    """Return query with appropriate base URIs.
+
+    If no base URIs are in the query add all the allowed ones.
+    If base URIs are provided only include the ones allowed.
+    """
+    return _preprocess_privileges(username, query)
+
+
+def search_datasets_by_user(username, query):
+    """Search the datasets the user has access to.
+
+    Valid keys for the query are: creator_usernames, base_uris, free_text.  If
+    the query dictionary is empty all datasets, that a user has access to, are
+    returned.
+
+    :param username: username
+    :param query: dictionary specifying query
+    :returns: List of dicts if user is valid and has access to datasets.
+              Empty list if user is valid but has not got access to any
+              datasets.
+    :raises: AuthenticationError if user is invalid.
+    """
+
+    query = preprocess_query_base_uris(username, query)
+    # If there are no base URIs at this point it means that the user is not
+    # allowed to search for anything.
     if len(query["base_uris"]) == 0:
         return []
 
@@ -305,6 +372,12 @@ def search_datasets_by_user(username, query):
         }
     )
     for ds in cx:
+
+        # Convert datetime object to float timestamp.
+        for key in ("created_at", "frozen_at"):
+            datetime_obj = ds[key]
+            ds[key] = dtoolcore.utils.timestamp(datetime_obj)
+
         datasets.append(ds)
     return datasets
 
@@ -570,9 +643,18 @@ def register_dataset(dataset_info):
             "Base URI is not registered: {}".format(base_uri)
         ))
 
+    try:
+        # Take a copy as register_dataset_descriptive_metadata makes
+        # changes to the dictionary, in particular it changes the
+        # types of the dates to datetime objects.
+        register_dataset_descriptive_metadata(dataset_info.copy())
+    except pymongo.errors.DocumentTooLarge as e:
+        raise(ValidationError(
+            "Dataset has too much metadata: {}".format(e)
+        ))
+
     if get_admin_metadata_from_uri(dataset_info["uri"]) is None:
         register_dataset_admin_metadata(dataset_info)
-    register_dataset_descriptive_metadata(dataset_info)
 
     return dataset_info["uri"]
 
